/*
 *            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
 *                    Version 2, December 2004
 *
 * Copyright (C) 2004 Sam Hocevar <sam@hocevar.net>
 *
 * Everyone is permitted to copy and distribute verbatim or modified
 * copies of this license document, and changing it is allowed as long
 * as the name is changed.
 *
 *            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
 *   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
 *
 *  0. You just DO WHAT THE FUCK YOU WANT TO.
 *
 */

#include <stdio.h>
#include <stdbool.h>
#include <cups/cups.h>

/* Usage function */
static void usage()
{
    fputs("usage: prout [-h] <file>\n", stdout);
}

/* Return the file name if the file is opennable. exit instead */
static char* check_file(char *file)
{

    FILE *f;

    /* Try to open the given file */
    f = fopen(file, "r");

    if (!f) {
        fputs("Cannot open the given file\n", stderr);
        exit (1);
    }

    /* Don't let the file close longer */
    fclose(f);

    return file;
}

/* Return a pointer to the destion if reachable. exit instead */
static cups_dest_t* check_dest()
{
    int num_dests;
    cups_dest_t *dests, *dest;

    /* Get the number of destionation configured */
    num_dests = cupsGetDests(&dests);

    /* Leave if there is no default printer configured */
    if (num_dests < 1) {
        fputs("No default printer configured\n", stderr);
        exit (1);
    }

    /* Get the default destination from all dests */
    dest = cupsGetDest(NULL, NULL, 1, dests);

    /* No need to go on if we can't reach the printer */
    if (!dest) {
        fputs("Cannot get a pointer to the default destination", stderr);
        exit (1);
    }

    return (dest);
}

/* Return the job id of the printed file */
static int print(const char* file, cups_dest_t *dest)
{
    int job_id = -1;

    /* Create a new job to print the document */
    job_id = cupsPrintFile (dest->name, file, "pp file", 0, NULL);

    fprintf(stdout, "printing %s (%d) to %s.\n", file, job_id, dest->name);

    return job_id;
}

/* Blocking function. Wait util the job is completed, or an error is thrown */
static void check_job (cups_dest_t *dest, int job_id)
{
    int i, num_jobs;

    cups_job_t *jobs;

    ipp_jstate_t state = IPP_JOB_PENDING;

    /* GET ALL THE JOBS "\_(°D°)/ */
    num_jobs = cupsGetJobs(&jobs, dest->name, 1, CUPS_WHICHJOBS_ALL);

    /* Find MY job id */
    for (i = 0; i < num_jobs; i++) {
        if (jobs[i].id == job_id) {
            state = jobs[i].state;
            break;
        }
    }

    /* Free the job array */
    cupsFreeJobs(num_jobs, jobs);

    switch (state) {
        case IPP_JOB_PENDING :
            fprintf(stdout, "job %d pending.\n", job_id);
            break;
        case IPP_JOB_HELD :
            fprintf(stdout, "job %d held.\n", job_id);
            break;
        case IPP_JOB_PROCESSING :
            fprintf(stdout, "job %d processing.\n", job_id);
            break;
        case IPP_JOB_STOPPED :
            fprintf(stdout, "job %d stopped.\n", job_id);
            break;
        case IPP_JOB_CANCELED :
            fprintf(stdout, "job %d canceled.\n", job_id);
            break;
        case IPP_JOB_ABORTED :
            fprintf(stdout, "job %d aborted.\n", job_id);
            break;
        case IPP_JOB_COMPLETED :
            fprintf(stdout, "job %d completed.\n", job_id);
            break;
    }

    return;
}

int main(int argc, char **argv)
{
    int i = 0;
    int job_id = -1;

    char *file;

    cups_dest_t *dest;

    /* check the given arguments */
    if (argc < 2) {
        usage();
        exit (1);
    }

    for (i=1; (i < argc) && (argv[i][0] == '-'); i++) {
        switch (argv[i][1]) {
            case 'h': usage(); exit(0);
            default:  usage(); exit(1);
        }
    }

    /* check if the file is correct */
    file = check_file(argv[1]);

    /* Is the default printer set up ? Can we use it ? */
    dest = check_dest();

    /* Well, let's print the file then */
    job_id = print(file, dest);

    /* How is the printing going ? */
    check_job(dest, job_id);

    return (0);
}
